<?php

namespace Drupal\bootstrap_theme_toggler\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;


/**
 * Provides a 'Theme Toggler' Block.
 *
 * @Block(
 *   id = "theme_toggler",
 *   admin_label = @Translation("Theme Toggler"),
 *   category = @Translation("Theme Toggler Block"),
 * )
 * 
 * When place, the machine name of the block instance will be enforced to be
 * the default name provided by Drupal, which should be the Block 'id' above,
 * without the underscore, 'themetoggler', prefixed by the default theme name
 * resulting in a machine name of 'loaded_default_theme_themetoggler'.
 * This is used to enforce only 1 instance of this block is placed on the site.
 */
class ThemeToggler extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->configuration;
    // \Drupal::messenger()->addMessage(json_encode($config), 'custom');
    // Build array of deafult bootstrap theme modes, if not disabled.
    if (!$config['custom_only']) {
      $bm = array("auto"=>"auto", "light"=>"light", "dark"=>"dark");
    } else {
      $bm = array();
    }
    // While the code executes without it, we need to check against a
    // null value, by checking for 'empty', to avoid a PHPUnit test error.
    if (!empty($config['color_modes'])) {
      if (!empty($bm)) {
        $themeModes = array_merge($bm, $config['color_modes']);
      } else {
        $themeModes = $config['color_modes'];
      }
    } else {
      $themeModes = $bm;
    }
    // \Drupal::messenger()->addMessage(json_encode($config), 'custom');
    return [
      '#theme' => 'block__toggler',
      '#attached' => [
        'library' => ['bootstrap_theme_toggler/theme-toggler'],
      ],
      '#toggler_label' => $config['toggler_label'],
      '#show_toggler_label' => $config['show_toggler_label'],
      '#show_option_labels' => $config['show_option_labels'],
      '#show_icons' => $config['show_icons'],
      '#show_checkmark' => $config['show_checkmark'],
      '#theme_modes' => $themeModes,
      // '#custom_only' => $config['custom_only'],
    ];
  }


  /**
   * Returns generic default configuration for block plugins.
   *
   * @return array
   *   An associative array with the default configuration.
   */
  protected function baseConfigurationDefaults() {
    return [
      'label' => 'Bootstrap Theme Toggler',
      'label_display' => 0,  //BlockPluginInterface::BLOCK_LABEL_VISIBLE,
      'provider' => $this->pluginDefinition['provider'],
      //   '#default_value' => false,
      //   '#return_value' => false,
    ];
  }


  // // NOT CURRENTLY USED - block validation checks the machine name to prevent duplicates
  // /**
  //  * function to determine if the Theme Toggler already exists in theme block layout.
  //  */
  // private function togglerBlockExists () {
  //   $togglerBlockFound = false;

  //   // Get a list of blocks that have been placed in regions for the default theme
  //   // Use the 'default' theme instead of the 'active' theme, which may be the admin theme.
  //   $themeName = \Drupal::config('system.theme')->get('default');
  //   // Get all blocks, visible and hidden/disabled.
  //   $regions = json_decode(json_encode(system_region_list($themeName, $show = REGIONS_ALL)), true);
  //   // $regionsVisible = \Drupal::service('block.repository')->getVisibleBlocksPerRegion();

  //   // DEBUGGING - $list is used for debugging to verify block names retrieved.
  //   // $list = [];
  //   foreach ($regions as $key => $region){
  //     $storage = \Drupal::entityTypeManager()->getStorage('block')->loadByProperties([
  //       // 'theme' =>  \Drupal::config('system.theme')->get('default'),
  //       'theme' => $themeName,
  //       'region'=> $key,
  //     ]);
  //     $blockTitle = array_key_first($storage);
  //     // Theme Toggler block is restricted to using the default machine name,
  //     // which is based on the block id without underscores, ie 'themetoggler'.
  //     if (str_contains($blockTitle ?? '', 'themetoggler')) {
  //       $togglerBlockFound = true;
  //     }
  //     // $list[$blockTitle] = $storage;
  //   }
  //   // DEBUGGING - using messages, because kint doesn't work on the block config form
  //   // \Drupal::messenger()->addMessage(json_encode($list), 'custom');
  //   // \Drupal::messenger()->addMessage(json_encode($storage), 'custom');
  //   // \Drupal::messenger()->addMessage(json_encode($regions), 'custom');
  //   // \Drupal::messenger()->addMessage($togglerBlockFound, 'custom');
  //   return $togglerBlockFound;
  // }


  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {

    $form['toggler_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Toggler Label'),
      '#description' => $this->t('Text to display on the toggler dropdown button.'),
      '#default_value' => $this->configuration['toggler_label'] ?? 'Theme',
    ];
    $form['show_toggler_label'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show toggler label'),
      // '#description' => $this->t('Check to make button label visible, on Toggler.'),
      '#default_value' => $this->configuration['show_toggler_label'] ?? true,
    ];
    $form['show_option_labels'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show theme mode option labels in dropdown list.'),
      // '#description' => $this->t('Check to make drop-down option labels visible, on Toggler.'),
      '#default_value' => $this->configuration['show_option_labels'] ?? true,
    ];
    $form['show_icons'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show toggler color mode icons'),
      '#description' => $this->t("To change icons being displayed, edit/replace the appropriate svg with the name of the theme mode to update. The svg element inside must be defined by a &lt;sybmol&gt; element, with an id matching the theme name. The filename should be the theme name prefixed by 'icon_'. <br>A replacement icon for the 'light' theme mode should have the tag, <code><xmp><symbol id='light'><path>...</path></symbol></xmp></code>and have the filename and location of <b>`bootstrap_theme_toggler/svg/icon_light.svg`</b>."),
      '#default_value' => $this->configuration['show_icons'] ?? true,
    ];
    $form['show_checkmark'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show a checkmark next to active theme'),
      // '#description' => $this->t('Check to show a checkmark next to active theme.'),
      '#default_value' => $this->configuration['show_checkmark'] ?? true,
    ];

    // DO NOT USE - the approach below, as this may retreive the admin theme
    // instead of the desired general site theme, which is the 'default' theme.
    // $themePath = \Drupal::service('theme.manager')->getActiveTheme()->getPath();

    // Get the the loaded 'default' theme, not the 'active' theme of the page.
    $themeRoot = \Drupal::config('system.theme')->getOriginal('default');
    $defaultTheme = \Drupal::service('extension.list.theme')->getPath($themeRoot).'/scss/';

    $prefix = 'theme_';
    $result = array();
    $defaultValues = array();

    // Look for any scss files inside the 'default_theme_root/scss' folder,
    // that has the prefix 'theme_'. It is assumed that if this folder exists,
    // that the theme is a bootstrap SASS based theme.
    // To add additional theme modes, create an appropriate scss file with the
    // name 'theme_modename' where, 'modename', is a lowercase theme mode ID.
    // EX: for a mode called 'blue', add 'theme_blue.scss' to the scss folder.
    $handle = @opendir($defaultTheme);
    if (!empty($handle)) {
      while ($entry = @readdir($handle)) {
        // If item is a file and has matching prefix, process as a theme mode.
        if(is_file($defaultTheme.$entry) && (mb_substr($entry, 0, 6) == $prefix)) {
          $result[str_replace($prefix, '', pathinfo($entry, PATHINFO_FILENAME))] = str_replace($prefix, '', pathinfo($entry, PATHINFO_FILENAME));
          // Building array of values only, for use in setting default_value.
          $defaultValues[] = str_replace($prefix, '', pathinfo($entry, PATHINFO_FILENAME));
        }
      }
      closedir($handle);
    }


    // Creates checkbox to enable/disable each custom scss theme modes found.
    if (!empty($result)){
      // $config = $this->configuration['color_modes'] ?? $defaultValues;
      $config = $this->configuration['color_modes'] ?? ($defaultValues ?? null);
      // For checkboxes, the array keys are used to set the checkbox values
      // and the array values are used to set the checkbox labels.
      $form['color_modes'] = array(
        '#type' => 'checkboxes',
        '#title' => $this->t('Custom Theme Modes Available'),
        // '#options' => array('blue' => $this->t('Blue'), 'red' => $this->t('Red')),
        '#options' => $result,
        '#description' => $this->t('Select the color theme modes to use on this toggler.'),
        '#default_value' => $config,
      );
    }
    $form['custom_only'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use only custom theme modes'),
      '#description' => $this->t("Checking this will disable 'light', 'dark' and 'auto' themes."),
      '#default_value' => $this->configuration['custom_only'] ?? false,
    ];

    // \Drupal::messenger()->addMessage(json_encode($this->configuration), 'custom');
    // $form['#validate'][] = 'validateForm';
    return $form;
  
  }  // END blockForm


  /**
   * {@inheritdoc}
   */
  public function blockValidate ($form, FormStateInterface $form_state) {
    
    // Check that the machine name matches the required default name.
    // Used to prevent more than one instance of this block being placed.

    // Get default theme name used to determine the block's machine name.
    $defaultTheme = \Drupal::config('system.theme')->getOriginal('default');
    // Get the actual machine name set for current block instance.
    $machine_name = $form_state->get('machine_name.initial_values');
    $defaultName = $machine_name['id'] == $defaultTheme.'_themetoggler';
    // $stored = $form_state->getStorage();

    // \Drupal::messenger()->addMessage(json_encode($machine_name), 'custom');
    // \Drupal::messenger()->addMessage(json_encode($defaultTheme), 'custom');
    // \Drupal::messenger()->addMessage(json_encode($stored), 'custom');

    if (!$defaultName) {
      $form_state->setErrorByName('machine_name', $this->
          t("Another instance of this block already exists, or the default machine name has been changed. <br> Make sure toggler machine name is 'DEFAULT_THEME_themetoggler', and that it has not already been placed into your site."));
    }
    $values = $form_state->getValues();
    // \Drupal::messenger()->addMessage(json_encode($values), 'custom');
    if ($values['custom_only']) {
      if (empty(array_filter($values['color_modes']))){
        $form_state->setErrorByName('color_modes', $this->
          t("Cannot select 'Custom Only' and have no custom themes enabled."));
      }
    }
  }


  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $values = $form_state->getValues();
    $this->configuration['color_modes'] = $values['color_modes'] ?? null;
    $this->configuration['toggler_label'] = $values['toggler_label'];
    $this->configuration['show_toggler_label'] = $values['show_toggler_label'];
    $this->configuration['show_option_labels'] = $values['show_option_labels'];
    $this->configuration['show_icons'] = $values['show_icons'];
    $this->configuration['show_checkmark'] = $values['show_checkmark'];
    $this->configuration['custom_only'] = $values['custom_only'];
    
    // \Drupal::messenger()->addMessage('Block saved', 'custom');
    // kint($values);
  }

}  // END of ThemeToggler class
